/*
 This is part of the OTF library. Copyright by ZIH, TU Dresden 2005-2006.
 Authors: Andreas Knuepfer, Holger Brunst, Ronny Brendel, Thomas Kriebitzsch
*/

#include "config.h"

#include <OTF_Writer.h>

#include "handler.h"
#include "hash.h"

#include <assert.h>


/* *** Definition handler *** ************************************* */

int OTF_handleDefinitionComment( void* fcb, uint32_t streamid,
	const char* comment ) {
	
	
	return ( 0 == OTF_Writer_writeDefinitionComment( ((fcbT*) fcb)->writer, streamid,
		comment ) );
}
	
int OTF_handleDefTimerResolution( void* fcbx,
	uint32_t streamid, uint64_t ticksPerSecond ) {
	
	
	fcbT *fcb = ((fcbT*) fcbx);
	streaminfoT* si = hash_search( fcb->hash, streamid );
	
	si->ticksPerSecond = ticksPerSecond;
	
	return ( 0 == OTF_Writer_writeDefTimerResolution( fcb->writer, streamid,
		ticksPerSecond ) );
}


int OTF_handleDefProcess( void* fcb, uint32_t streamid
		, uint32_t deftoken, const char* name, uint32_t parent ) {


	return ( 0 == OTF_Writer_writeDefProcess( ((fcbT*) fcb)->writer, streamid,
		deftoken, name, parent ) );
}


int OTF_handleDefProcessGroup( void* fcb, uint32_t streamid
		, uint32_t deftoken, const char* name, uint32_t n, uint32_t* array ) {


	return ( 0 == OTF_Writer_writeDefProcessGroup( ((fcbT*) fcb)->writer, streamid, deftoken,
		name, n, array ) );
}


int OTF_handleDefFunction(  void* fcbx, uint32_t streamid,
		uint32_t deftoken, const char* name, 
		uint32_t group, uint32_t scltoken ) {


	fcbT *fcb = ((fcbT*) fcbx);
	streaminfoT* si = hash_search( fcb->hash, streamid );
	
	/* allocate new memory if necessary */
	if ( si->nfunctions >= si->sfunctions )
	{
		si->sfunctions = ( si->sfunctions > 0 ) ? ( 2* si->sfunctions ) : 20;
		si->functions = (functionT*) realloc( si->functions,
			si->sfunctions * sizeof(functionT) );
			
		assert( NULL != si->functions );
	}

	/* insert all data about the function */
	si->functions[si->nfunctions].deftoken = deftoken;
	si->functions[si->nfunctions].name = strdup( name );
	si->functions[si->nfunctions].group = group;
	si->functions[si->nfunctions].scltoken = scltoken;

	++(si->nfunctions);

	
	return ( 0 == OTF_Writer_writeDefFunction( fcb->writer, streamid, deftoken,
		name, group, scltoken ) );
}


int OTF_handleDefFunctionGroup( void* fcbx, uint32_t streamid,
		uint32_t deftoken, const char* name ) {


	fcbT *fcb = ((fcbT*) fcbx);
	streaminfoT* si = hash_search( fcb->hash, streamid );
	
	/* allocate new memory if necessary */
	if ( si->nfunctiongroups >= si->sfunctiongroups )
	{
		si->sfunctiongroups = ( si->sfunctiongroups > 0 )
			? ( 2* si->sfunctiongroups ) : 10;
		si->functiongroups = (functiongroupT*) realloc( si->functiongroups,
			si->sfunctiongroups * sizeof(functiongroupT) );
			
		assert( NULL != si->functiongroups );
	}

	/* insert all data about the function */
	si->functiongroups[si->nfunctiongroups].deftoken = deftoken;
	si->functiongroups[si->nfunctiongroups].name = strdup( name );

	++(si->nfunctiongroups);
	
	return ( 0 == OTF_Writer_writeDefFunctionGroup( fcb->writer, streamid,
		deftoken, name ) );
}


int OTF_handleDefCounter( void* fcb, uint32_t streamid,
		uint32_t deftoken, const char* name, uint32_t properties, 
		uint32_t countergroup, const char* unit ) {


	return ( 0 == OTF_Writer_writeDefCounter( ((fcbT*) fcb)->writer, streamid,
		deftoken, name, properties, countergroup, unit ) );
}


int OTF_handleDefCounterGroup( void* fcb, uint32_t streamid,
		uint32_t deftoken, const char* name ) {


	return ( 0 == OTF_Writer_writeDefCounterGroup( ((fcbT*) fcb)->writer, streamid, 
			deftoken, name ) );
}


int OTF_handleDefScl(  void* fcb, uint32_t streamid, 
		uint32_t deftoken, uint32_t sclfile, uint32_t sclline ) {


	return ( 0 == OTF_Writer_writeDefScl( ((fcbT*) fcb)->writer, streamid, deftoken,
		sclfile, sclline ) );
}


int OTF_handleDefSclFile(  void* fcb, uint32_t streamid,
		uint32_t deftoken, const char* filename ) {


	return ( 0 == OTF_Writer_writeDefSclFile( ((fcbT*) fcb)->writer, streamid, 
		deftoken, filename ) );
}


int OTF_handleDefVersion( void* fcb, uint32_t streamid, 
		uint8_t major, uint8_t minor, uint8_t sub, const char* string ) {


	/* deprecated */

	assert( 0 );

	return 0;

	/*
	return OTF_Writer_writeDefVersion( ((fcbT*) fcb)->writer, 
		major, minor, sub, string );
	*/
}


int OTF_handleDefCreator( void* fcb, uint32_t streamid,
		const char* creator ) {


	return ( 0 == OTF_Writer_writeDefCreator( ((fcbT*) fcb)->writer, streamid,
		creator ) );
}


/* deprecated record types */
#if 0


int OTF_handleDefiofilename( void* fcb, uint32_t streamid,
		uint32_t deftoken, const char* iofilename, uint32_t communicator ) {


	return ( 0 == OTF_Writer_writeDefiofile( ((fcbT*) fcb)->writer, streamid, deftoken,
		communicator, iofilename ) );
}


int OTF_handleDefopenmpname( void* fcb, uint32_t streamid, 
		uint32_t deftoken, const char* name ) {


	return ( 0 == OTF_Writer_writeDefopenmpname( ((fcbT*) fcb)->writer, streamid,
		deftoken, name ) );
}


int OTF_handleDefopenmpgroup( void* fcb, uint32_t streamid,
		uint32_t deftoken, const char* name ) {


	return ( 0 == OTF_Writer_writeDefopenmpgroup( ((fcbT*) fcb)->writer, streamid,
		deftoken, name ) );
}

#endif /* 0 */


/* *** Event handler *** ****************************************** */


int OTF_handleEventComment( void* fcb, uint64_t time, 
		const char* comment ) {


	return ( 0 == OTF_Writer_writeEventComment( ((fcbT*) fcb)->writer, time, 0,
		comment ) );
}


int OTF_handleCounter( void* fcb, uint64_t time, 
		uint32_t process, uint32_t counter_token, uint64_t value ) {


	return ( 0 == OTF_Writer_writeCounter( ((fcbT*) fcb)->writer, time, 
		process, counter_token, value ) );
}


int OTF_handleEnter( void* fcb, uint64_t time, 
		uint32_t statetoken, uint32_t cpuid, uint32_t scltoken ) {


	return ( 0 == OTF_Writer_writeEnter( ((fcbT*) fcb)->writer, time, 
		statetoken,	cpuid, scltoken ) );
}


/* deprecated record types */
#if 0

int OTF_handleFileiobegin( void* fcb, uint64_t time, 
		uint32_t process, uint64_t tag, uint8_t type, uint32_t iofiletoken, 
		uint64_t size, uint32_t scltoken ) {


	return ( 0 == OTF_Writer_writeFileiobegin( ((fcbT*) fcb)->writer, time, process,
		tag, type, iofiletoken, size, scltoken ) );
}


int OTF_handleFileioend( void* fcb, uint64_t time, 
		uint32_t process, uint64_t tag, uint8_t type, uint32_t iofiletoken, 
		uint64_t size, uint32_t scltoken ) {


	return ( 0 == OTF_Writer_writeFileioend( ((fcbT*) fcb)->writer, time, process,
		tag, type, iofiletoken, size, scltoken ) );
}

#endif /* 0 */


int OTF_handleCollectiveOperation( void* fcb, uint64_t time, 
    	uint32_t process, uint32_t functionToken, uint32_t communicator, 
    	uint32_t rootprocess, uint32_t sent, uint32_t received, 
    	uint64_t duration, uint32_t scltoken ) {


    return ( 0 == OTF_Writer_writeCollectiveOperation( ((fcbT*) fcb)->writer, time, 
    	process, functionToken, communicator, rootprocess, 
		sent, received, duration, scltoken ) );
}

/* deprecated record types */
#if 0

int OTF_handleMutexacquire( void* fcb, uint64_t time, 
		uint32_t process, uint64_t token, uint64_t duration ) {


	return ( 0 == OTF_Writer_writeMutexacquire( ((fcbT*) fcb)->writer, time, process,
		token, duration ) );
}


int OTF_handleMutexrelease( void* fcb, uint64_t time, 
		uint32_t process, uint64_t token ) {


	return ( 0 == OTF_Writer_writeMutexrelease( ((fcbT*) fcb)->writer, time, 
		process, token ) );
}


int OTF_handleOpenmpenter( void* fcb, uint64_t time, 
		uint32_t process, uint32_t name, uint32_t type, uint32_t scltoken ) {


	return ( 0 == OTF_Writer_writeOpenmpenter( ((fcbT*) fcb)->writer, time, 
		process, name, type, scltoken ) );
}


int OTF_handleOpenmpleave( void* fcb, uint64_t time, 
		uint32_t process, uint32_t scltoken ) {


	return ( 0 == OTF_Writer_writeOpenmpleave( ((fcbT*) fcb)->writer, time, 
		process, scltoken ) );
}

#endif /* 0 */

int OTF_handleRecvMsg( void* fcb, uint64_t time, 
		uint32_t receiver, uint32_t sender, uint32_t communicator, 
		uint32_t msgtype, uint32_t msglength, uint32_t scltoken ) {


	return ( 0 == OTF_Writer_writeRecvMsg( ((fcbT*) fcb)->writer, time, receiver,
		sender, communicator, msgtype, msglength, scltoken ) );
}


int OTF_handleSendMsg( void* fcb, uint64_t time, 
		uint32_t sender, uint32_t receiver, uint32_t communicator, 
		uint32_t msgtype, uint32_t msglength, uint32_t scltoken ) {


	return ( 0 == OTF_Writer_writeSendMsg( ((fcbT*) fcb)->writer, time, sender,
		receiver, communicator, msgtype, msglength, scltoken ) );
}


int OTF_handleLeave( void* fcb, uint64_t time, 
		uint32_t statetoken, uint32_t cpuid, uint32_t scltoken ) {


	return ( 0 == OTF_Writer_writeLeave( ((fcbT*) fcb)->writer, time, statetoken,
		cpuid, scltoken ) );
}


int OTF_handleBeginProcess( void* fcb, uint64_t time,
		uint32_t cpuid ) {


	return ( 0 == OTF_Writer_writeBeginProcess( ((fcbT*) fcb)->writer, time, cpuid ) );
}
int OTF_handleEndProcess( void* fcb, uint64_t time,
		uint32_t cpuid ) {


	return ( 0 == OTF_Writer_writeEndProcess( ((fcbT*) fcb)->writer, time, cpuid ) );
}


/* *** snapshot handlers ********************************************** */


int OTF_handleEnterSnapshot( void *fcb, uint64_t time,
	uint64_t originaltime, uint32_t function, uint32_t process,
	uint32_t source ) {


	return ( 0 == OTF_Writer_writeEnterSnapshot( ((fcbT*) fcb)->writer,
		time, originaltime, function, process, source ) );
}


int OTF_handleSendSnapshot( void *fcb, uint64_t time,
	uint64_t originaltime, uint32_t sender, uint32_t receiver,
	uint32_t procGroup, uint32_t tag, uint32_t source ) {
	

	return ( 0 == OTF_Writer_writeSendSnapshot( ((fcbT*) fcb)->writer,
		time, originaltime, sender, receiver, procGroup, tag, source ) );
}


/* *** Summary handlers *** ****************************************** */


int OTF_handleFunctionSummary( void* fcb, 
		uint64_t time, uint32_t function, uint32_t process, 
		uint64_t count, uint64_t excltime, uint64_t incltime ) {


	return ( 0 == OTF_Writer_writeFunctionSummary( ((fcbT*) fcb)->writer, 
		time, function, process, count, excltime, incltime ) );
}


int OTF_handleFunctionGroupSummary( void* fcb, 
		uint64_t time,  uint32_t functiongroup,  uint32_t process,  
		uint64_t count,  uint64_t excltime,  uint64_t incltime ) {


	return ( 0 == OTF_Writer_writeFunctionGroupSummary( ((fcbT*) fcb)->writer, 
		time, functiongroup, process, count, excltime, incltime ) );
}


int OTF_handleMessageSummary( void* fcb, 
		uint64_t time, uint32_t process, uint32_t peer, 
		uint32_t comm, uint32_t tag, uint64_t number_sent, 
		uint64_t number_recvd, uint64_t bytes_sent, uint64_t bytes_recved ) {


	return ( 0 == OTF_Writer_writeMessageSummary( ((fcbT*) fcb)->writer, 
		time, process, peer, comm, tag, number_sent, number_recvd, bytes_sent,
		bytes_recved ) );
}


int OTF_handleUnknown( void* fcb, uint64_t time, uint32_t process, const char* record ) {

#ifdef OTF_VERBOSE

	if ( (uint64_t) -1 != time ) {

		printf( "  unknown record >%s< at process 0x%x time 0x%llx\n", 
			record, process, (unsigned long long) time );

	} else {

		printf( "  unknown record >%s< at stream 0x%x\n", 
			record, process );
	}

#endif /* OTF_VERBOSE */

	return 1;
}

