/*
 This is part of the OTF library. Copyright by ZIH, TU Dresden 2005-2006.
 Authors: Andreas Knuepfer, Holger Brunst, Ronny Brendel, Thomas Kriebitzsch
*/

#ifndef HANDLER_H
#define HANDLER_H

#include <inttypes.h>
#include <OTF_Writer.h>

typedef struct {

	uint32_t deftoken;
	char *name;
	uint32_t group;
	uint32_t scltoken;
}functionT;

typedef struct {

	uint32_t deftoken;
	char* name;
}functiongroupT;

typedef struct {

	uint32_t streamid;

	uint64_t ticksPerSecond;
	
	functionT *functions;
	int nfunctions; /* number of functions in the functioninfo-array */
	int sfunctions; /* size of the functioninfo-array */
	functiongroupT *functiongroups;
	int nfunctiongroups; /* number of groups in the functiongroup-array */
	int sfunctiongroups; /* size of the functiongroups-array */
	
}streaminfoT;

typedef struct hashtabS
{
	streaminfoT entry;
	
	int entryvecsize;
	streaminfoT *p_entryvec;
}hashtabT;

typedef struct {

	OTF_Writer *writer;
	hashtabT *hash;
	int nstreaminfos; /* number of streams in the  streaminfos-array */
	
}fcbT;

/* *** Definition handler *** ************************************* */

int OTF_handleDefinitionComment( void* firsthandlerarg, uint32_t streamid,
	const char* comment );
	
int OTF_handleDefTimerResolution( void* firsthandlerarg,
	uint32_t streamid, uint64_t ticksPerSecond );

int OTF_handleDefProcess( void* firsthandlerarg, uint32_t streamid,
	uint32_t deftoken, const char* name, uint32_t parent );

int OTF_handleDefProcessGroup( void* firsthandlerarg, uint32_t streamid,
	uint32_t deftoken, const char* name, uint32_t n, uint32_t* array );

int OTF_handleDefFunction( void* firsthandlerarg, uint32_t streamid,
	uint32_t deftoken, const char* name, uint32_t group, uint32_t scltoken );

int OTF_handleDefFunctionGroup( void* firsthandlerarg, uint32_t streamid,
	uint32_t deftoken, const char* name );

int OTF_handleDefCounter( void* firsthandlerarg, uint32_t streamid,
	uint32_t deftoken, const char* name, uint32_t properties, 
	uint32_t countergroup, const char* unit );

int OTF_handleDefCounterGroup( void* firsthandlerarg, uint32_t streamid,
	uint32_t deftoken, const char* name );

int OTF_handleDefScl( void* firsthandlerarg, uint32_t streamid,
	uint32_t deftoken, uint32_t sclfile, uint32_t sclline );

int OTF_handleDefSclFile( void* firsthandlerarg, uint32_t streamid,
	uint32_t deftoken, const char* filename );

int OTF_handleDefversion( void* firsthandlerarg, uint32_t streamid,
	uint8_t major, uint8_t minor, uint8_t sub, const char* string );

int OTF_handleDefCreator( void* firsthandlerarg, uint32_t streamid,
	const char* creator );
/*
int OTF_handleDefiofilename( void* firsthandlerarg, uint32_t streamid,
	uint32_t deftoken, const char* iofilename, uint32_t communicator );

int OTF_handleDefopenmpname( void* firsthandlerarg, uint32_t streamid,
	uint32_t deftoken, const char* name );

int OTF_handleDefopenmpgroup( void* firsthandlerarg, uint32_t streamid,
	uint32_t deftoken, const char* name );
*/
/* *** Event handler *** ****************************************** */

int OTF_handleEventComment( void* firsthandlerarg, uint64_t time, 
	const char* comment );

int OTF_handleCounter( void* firsthandlerarg, uint64_t time, 
	uint32_t process, uint32_t counter_token, uint64_t value );

int OTF_handleEnter( void* firsthandlerarg, uint64_t time, 
	uint32_t statetoken, uint32_t cpuid, uint32_t scltoken );
/*
int OTF_handleFileiobegin( void* firsthandlerarg, uint64_t time, 
	uint32_t process, uint64_t tag, uint8_t type, uint32_t iofiletoken, 
	uint64_t size, uint32_t scltoken );

int OTF_handleFileioend( void* firsthandlerarg, uint64_t time, 
	uint32_t process, uint64_t tag, uint8_t type, uint32_t iofiletoken, 
	uint64_t size, uint32_t scltoken );
*/
int OTF_handleCollectiveOperation( void* firsthandlerarg, uint64_t time, 
    uint32_t process, uint32_t functionToken, uint32_t communicator, 
    uint32_t rootprocess, uint32_t sent, uint32_t received, 
    uint64_t duration, uint32_t scltoken );
/*
int OTF_handleMutexacquire( void* firsthandlerarg, uint64_t time, 
	uint32_t process, uint64_t token, uint64_t duration );

int OTF_handleMutexrelease( void* firsthandlerarg, uint64_t time, 
	uint32_t process, uint64_t token );

int OTF_handleOpenmpenter( void* firsthandlerarg, uint64_t time, 
	uint32_t process, uint32_t name, uint32_t type, uint32_t scltoken );

int OTF_handleOpenmpleave( void* firsthandlerarg, uint64_t time, 
	uint32_t process, uint32_t scltoken );
*/
int OTF_handleRecvMsg( void* firsthandlerarg, uint64_t time, 
	uint32_t receiver, uint32_t sender, uint32_t communicator, 
	uint32_t msgtype, uint32_t msglength,
	uint32_t scltoken );

int OTF_handleSendMsg( void* firsthandlerarg, uint64_t time, 
	uint32_t sender, uint32_t receiver, uint32_t communicator, 
	uint32_t msgtype, uint32_t msglength, uint32_t scltoken );

int OTF_handleLeave( void* firsthandlerarg, uint64_t time, 
	uint32_t statetoken, uint32_t cpuid, uint32_t scltoken );

int OTF_handleBeginProcess( void* firsthandlerarg, uint64_t time,
	uint32_t process );

int OTF_handleEndProcess( void* firsthandlerarg, uint64_t time,
	uint32_t process );


/* *** Handlers for OTF snapshot records ****************************** */


int OTF_handleEnterSnapshot( void *firsthandlerarg, uint64_t time,
	uint64_t originaltime, uint32_t function, uint32_t process,
	uint32_t source );


int OTF_handleSendSnapshot( void *firsthandlerarg, uint64_t time,
	uint64_t originaltime, uint32_t sender, uint32_t receiver,
	uint32_t procGroup, uint32_t tag, uint32_t source );
	
	
/* *** Summary handler *** ****************************************** */

int OTF_handleFunctionSummary( void* firsthandlerarg, 
	uint64_t time, uint32_t function, uint32_t process, 
	uint64_t count, uint64_t excltime, uint64_t incltime );

int OTF_handleFunctionGroupSummary( void* firsthandlerarg, 
	uint64_t time, uint32_t functiongroup, uint32_t process, 
	uint64_t count, uint64_t excltime, uint64_t incltime );

int OTF_handleMessageSummary( void* firsthandlerarg, 
	uint64_t time, uint32_t process, uint32_t peer, 
	uint32_t comm, uint32_t tag, uint64_t number_sent, uint64_t number_recvd,
	uint64_t bytes_sent, uint64_t bytes_recved );


/* *** Misc handlers *** ******************************************** */

int OTF_handleUnknown( void* fcb, uint64_t time, uint32_t process, const char* record );


#endif /* OTF_handleH */
